# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""ASTER_BUILD class.
"""

__revision__ = "$Id: as_build.py,v 1.53 2005/10/20 15:52:32 mcourtoi Exp $"
__cvstag__ = "$Name: v1_3_0 $"

# ----- differ messages translation
def _(mesg):
   return mesg

import os
import glob
import re
import copy
import time
import py_compile
from types import ListType, TupleType

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_BUILD:
   """This class provides functions to build a version of Code_Aster.
   """
   repcatapy = ('entete', 'commun', 'commande')
   repcatalo = ('compelem', 'typelem', 'options')

#-------------------------------------------------------------------------------
   def __init__(self, run, conf):
      """run   : ASTER_RUN object
      conf : ASTER_CONFIG object
      """
      # initialisations
      self.verbose = False
      self.debug = False
      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      if run != None and run.__class__.__name__ == 'ASTER_RUN':
         self.verbose = run['verbose']
         self.debug = run['debug']
      # ----- Check if 'conf' is a valid ASTER_CONFIG object
      self.conf = conf
      if conf == None or conf.__class__.__name__ != 'ASTER_CONFIG':
         self._mess(_('no configuration object provided !'), _('<F>_ERROR'))
      else:
         # ----- check that ASTER_CONFIG fields are correct (not nul !)
         pass

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def Compil(self, typ, rep, repobj, dbg, rep_trav=''):
      """Compile 'rep/*.suffix' files and put '.o' files in 'repobj'
         dbg : debug or nodebug
         (rep can also be a file)
      rep can be remote.
      """
      prev = os.getcwd()
      # ----- get command line from conf object
      if typ == 'C':
         suffix = '.c'
         if dbg == 'debug':
            opts = self.conf['OPTC_D'][0]
         else:
            opts = self.conf['OPTC_O'][0]
         if self.conf['CC'][0] != '':
            cmd = self.conf['CC'][0]+' '+opts+' '+' '.join(self.conf['INCL'])
         else:
            self._mess(_("C compiler not defined in 'config.txt' (CC)"), \
                  _('<F>_CMD_NOT_FOUND'))
      elif typ == 'F':
         suffix = '.f'
         if dbg == 'debug':
            opts = self.conf['OPTF_D'][0]
         else:
            opts = self.conf['OPTF_O'][0]
         if self.conf['F77'][0] != '':
            cmd = self.conf['F77'][0]+' '+opts+' '+' '.join(self.conf['INCLF'])
         else:
            self._mess(_("Fortran 77 compiler not defined in 'config.txt' (F77)"), \
                  _('<F>_CMD_NOT_FOUND'))
      elif typ == 'F90':
         suffix = '.f'
         if dbg == 'debug':
            opts = self.conf['OPTF90_D'][0]
         else:
            opts = self.conf['OPTF90_O'][0]
         if self.conf['F90'][0] != '':
            cmd = self.conf['F90'][0]+' '+opts+' '+' '.join(self.conf['INCLF90'])
         else:
            self._mess(_("Fortran 90 compiler not defined in 'config.txt' (F90)"), \
                  _('<F>_CMD_NOT_FOUND'))
      else:
         self._mess(_('unknown type : %s') % typ, _('<F>_PROGRAM_ERROR'))
      
      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']
      
      # ----- copy source files if remote
      if self.run.IsRemote(rep):
         if rep_trav == '':
            self.run.Mess(_('rep_trav must be defined'), _('<F>_PROGRAM_ERROR'))
         obj = os.path.join(rep_trav, '__tmp__'+os.path.basename(rep))
         iret = self.run.MkDir(obj)
         if self.run.IsDir(rep):
            src = os.path.join(rep, '*'+suffix)
         else:
            src = rep
         iret = self.run.Copy(obj, src)
         rep = obj
      else:
         rep = self.run.PathOnly(rep)

      # ----- check that directories exist
      if not os.path.isdir(repobj):
         iret = self.run.MkDir(repobj, verbose=True)
      
      # ----- work in repobj
      os.chdir(repobj)

      # ----- log file
      msg = os.path.basename(rep)+'.msg'
      if os.path.exists(msg):
         os.remove(msg)
      fmsg = open(msg, 'w')
      fmsg.write(os.linesep + \
                 'Messages de compilation' + os.linesep + \
                 '=======================' + os.linesep)

      # ----- list of source files
      if self.run.IsDir(rep):
         files = glob.glob(os.path.join(rep, '*'+suffix))
      else:
         files = [rep, ]
      if len(files) == 0:
         self._mess(_('no source file in %s') % rep)
         return 0, []
      iret = 0
      nskip = 0
      for f in files:
         tail = os.path.join('...', os.path.basename(os.path.dirname(f)),
                                  os.path.basename(f))
         obj = os.path.basename(f)[:-2]+'.o'

         # ----- skip file if .o more recent than source file
         if not force and os.path.exists(obj) \
               and os.stat(obj).st_ctime > os.stat(f).st_ctime:
            nskip += 1
            if self.verbose or self.debug:
               self.run.VerbStart(_('compiling %s') % tail, verbose=True)
               self.run.VerbIgnore(verbose=True)
         else:
            self.run.VerbStart(_('compiling %s') % tail, verbose=True)
            if self.verbose:
               print
            jret, out = self.run.Shell(cmd+' '+f)
            # ----- avoid to duplicate output
            if not self.verbose:
               self.run.VerbEnd(jret, output=out, verbose=True)
            if jret != 0:
               fmsg.write(out)
               self._mess((_('error during compiling %s (see %s)') % \
                     (f, os.path.abspath(msg))),
                     _('<E>_COMPIL_ERROR'), store=True)
            iret = max(iret, jret)
      self._mess(_('%4d files to compile in %s') % (len(files), rep))
      if nskip > 0:
         self._mess(_('%4d files skipped (object more recent than source)') \
                % nskip)

      fmsg.close()
      os.chdir(prev)
      return iret, files

#-------------------------------------------------------------------------------
   def CompilAster(self, REPREF, repdest='', dbg='nodebug', ignore_ferm=False):
      """Compile all Aster source files, put '.o' files in 'repdest'/DIR
      and DIR_f with DIR=obj or dbg depends on 'dbg'
      (DIR for "real" source files and DIR_f for fermetur).
      """
      self._mess(_('Compilation of source files'), 'TITLE')
      dest_o = {  'nodebug' : self.conf['BINOBJ_NODBG'][0],
                  'debug'   : self.conf['BINOBJ_DBG'][0]    }
      dest_f_o = {'nodebug' : self.conf['BINOBJF_NODBG'][0],
                  'debug'   : self.conf['BINOBJF_DBG'][0]   }

      # ----- define type and destination of '*.o' files for each directory
      if repdest == '':
         repdest = REPREF
      dest = os.path.join(repdest, dest_o[dbg])
      dest_f = os.path.join(repdest, dest_f_o[dbg])
      para = {
         self.conf['SRCC'][0]     : ['C', dest],
         self.conf['SRCFOR'][0]   : ['F', dest],
         self.conf['SRCF90'][0]   : ['F90', dest],
      }
      if not ignore_ferm:
         para[self.conf['SRCFERM'][0]] = ['F', dest_f]

      # ----- check that directories exist
      for obj in [os.path.join(REPREF, rep) for rep in para.keys()]:
         if not os.path.exists(obj):
            self._mess(_('directory does not exist : %s') % obj,
                  _('<E>_FILE_NOT_FOUND'))
      self.run.CheckOK()

      # ----- nobuild list
      nobuild = []
      for d in self.conf['NOBUILD']:
         nobuild.extend([os.path.join(REPREF, d) for d in d.split()])
      # bibf90 dirs automatically added if there is no F90 compiler
      if self.conf['F90'][0] == '' and self.conf['SRCF90'][0] != '':
         nobuild.extend([d for d in \
            glob.glob(os.path.join(REPREF, self.conf['SRCF90'][0], '*'))
            if not d in nobuild])
         self._mess(_("There is no Fortran 90 compiler in 'config.txt'. " \
               "All source files from '%s' were ignored." % \
               self.conf['SRCF90'][0]), '<A>_IGNORE_F90', store=True)
      if len(nobuild)>0:
         self._mess(_('These directories will not be built :'))
         for d in nobuild:
            print ' '*10+'>>> %s' % d
      
      # ----- compilation
      iret = 0
      for module in [d for d in para.keys() if d != '']:
         if module != self.conf['SRCFERM'][0]:
            # ----- glob subdirectories
            lrep = glob.glob(os.path.join(REPREF, module, '*'))
         else:
            lrep = (os.path.join(REPREF, module), )
         for rep in lrep:
            if os.path.isdir(rep):
               tail = '...'+re.sub('^'+REPREF, '', rep)
               print
               jret = 0
               if not rep in nobuild:
                  self._mess(_('Compilation from %s directory') % tail)
                  jret, bid = self.Compil(para[module][0], rep, para[module][1], dbg)
               else:
                  self.run.VerbStart(_('Directory %s is in NOBUILD list') % tail, \
                        verbose=True)
                  self.run.VerbIgnore(verbose=True)
               iret = max(iret, jret)

      return iret

#-------------------------------------------------------------------------------
   def Archive(self, repobj, lib):
      """Archive all .o files from 'repobj' into 'lib'
      All args must be local.
      """
      self._mess(_('Archive object files'), 'TITLE')
      prev = os.getcwd()
      # ----- get command line from conf object
      cmd0 = [self.conf['LIB'][0]+' '+lib]

      # ----- check that directories exist
      if not os.path.isdir(repobj):
         self._mess(_('directory does not exist : %s') % repobj,
               _('<E>_FILE_NOT_FOUND'))
      rep = os.path.dirname(lib)
      if not os.path.isdir(rep):
         iret = self.run.MkDir(rep, verbose=True)
      self.run.CheckOK()

      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']
      if not os.path.exists(lib):
         force = True
         tlib = -1
      else:
         tlib = os.stat(lib).st_ctime

      # ----- work in repobj
      os.chdir(repobj)

      # ----- list of source files
      files = glob.glob('*.o')
      ntot = len(files)
      if ntot == 0:
         self._mess(_('no object file in %s') % rep)
         return 0
      
      # ----- sleep 1 s to be sure than last .o will be older than lib !
      time.sleep(1)

      # ----- use MaxCmdLen to limit command length
      iret = 0
      nskip = 0
      while len(files) > 0:
         cmd = copy.copy(cmd0)
         clen = len(cmd[0])+1
         nadd = 0
         while len(files) > 0 and clen+len(files[0]) < self.run.MaxCmdLen-15:
            if not force and tlib > os.stat(files[0]).st_ctime:
               nskip += 1
               bid = files.pop(0)
            else:
               clen = clen+len(files[0])+1
               cmd.append(files.pop(0))
               nadd += 1
         self.run.VerbStart(_('%4d / %4d objects added to archive...') % \
               (nadd, ntot), verbose=True)
         if nadd > 0:
            if self.verbose:
               print
            jret, out = self.run.Shell(' '.join(cmd))
            # ----- avoid to duplicate output
            if not self.verbose:
               self.run.VerbEnd(jret, output=out, verbose=True)
            if jret != 0:
               self._mess(_('error during archiving'), _('<E>_ARCHIVE_ERROR'))
            iret = max(iret, jret)
         else:
            self.run.VerbIgnore(verbose=True)

      self._mess(_('%4d files archived') % ntot)
      if nskip > 0:
         self._mess(_('%4d files skipped (objects older than library)') \
                % nskip)
      os.chdir(prev)
      return iret

#-------------------------------------------------------------------------------
   def Link(self, exe, lobj, libaster, libferm, reptrav):
      """Link a number of object and archive files into an executable.
         exe      : name of the executable to build
         lobj     : list of object files
         libaster : Code_Aster main library
         libferm  : Code_Aster fermetur library
         reptrav  : working directory
      If "python.o" is not in 'lobj' it will be extracted from 'libaster'.
      Other libs are given by ASTER_CONFIG object.
      None argument must not be remote !
      """
      self._mess(_('Build Code_Aster executable'), 'TITLE')
      prev = os.getcwd()
      # ----- check that directories exist
      for obj in lobj+[libaster, libferm]:
         if not os.path.exists(obj):
            self._mess(_('file not found : %s') % obj, _('<E>_FILE_NOT_FOUND'))
      rep = os.path.dirname(exe)
      if not os.path.isdir(rep):
         iret = self.run.MkDir(rep, verbose=True)
      self.run.CheckOK()

      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']
      if not os.path.exists(exe):
         force = True
      else:
         texe = os.stat(exe).st_ctime
         for f in lobj+[libaster, libferm]:
            if os.stat(f).st_ctime > texe:
               force = True
               if self.debug:
                  print '<DBG> %s more recent than %s' % (f, exe)
               break

      # ----- work in reptrav
      os.chdir(reptrav)

      # ----- python.o in lobj ?
      mainobj = self.conf['BINOBJ_MAIN'][0]
      if not mainobj in [os.path.basename(o) for o in lobj]:
         # get 'ar' command without arguments, if it's not 'ar' &$#@!
         cmd = [self.conf['LIB'][0].split()[0]]
         cmd.extend(['-xv', libaster, mainobj])
         cmd = ' '.join(cmd)
         self.run.VerbStart(_('extracting %s from %s...') % \
               (repr(mainobj), os.path.basename(libaster)), verbose=True)
         if self.verbose:
            print
         iret, out = self.run.Shell(cmd)
         # ----- avoid to duplicate output
         if not self.verbose:
            self.run.VerbEnd(iret, output=out, verbose=True)
         if iret != 0:
            self._mess(_('error during extracting %s') % repr(mainobj),
                  _('<F>_LIBRARY_ERROR'))
         lobj.insert(0, mainobj)
      else:
         lobj = [o for o in lobj if os.path.basename(o) == mainobj] +\
                [o for o in lobj if os.path.basename(o) != mainobj]

      # ----- get command line from conf object and args
      cmd = copy.copy(self.conf['LINK'])
      cmd.extend(self.conf['OPTL'])
      cmd.append('-o')
      cmd.append(exe)
      cmd.extend(lobj)
      cmd.append(libaster)
      cmd.extend(self.conf['BIBL'])
      cmd.append(libferm)
      cmd = ' '.join(cmd)

      # ----- run ld or skip this stage
      iret = 0
      tail = os.path.join('...', os.path.basename(rep), os.path.basename(exe))
      self.run.VerbStart(_('creating %s...') % tail, verbose=True)
      if force:
         if self.verbose:
            print
         iret, out = self.run.Shell(cmd)
         # ----- avoid to duplicate output
         if not self.verbose:
            self.run.VerbEnd(iret, output=out, verbose=True)
         if iret != 0:
            self._mess(_('error during linking'), _('<E>_LINK_ERROR'))
      else:
         self.run.VerbIgnore(verbose=True)
         self._mess(_('executable more recent than objects and libs ' \
               'in arguments'))

      os.chdir(prev)
      return iret

#-------------------------------------------------------------------------------
   def PrepEnv(self, REPREF, repdest, dbg='nodebug', **kargs):
      """Prepare 'repdest' with Code_Aster environment from REPREF and
      given arguments in kargs[k] (k in 'exe', 'cmde', 'ele', 'py', 'unigest').
      Note : - REPREF/elements and kargs['ele'] don't exist when building
         elements for the first time.
             - only PYSUPPR entries from 'unigest' are considered here.
      All elements of kargs can be remote.
      """      
      reptrav = os.path.join(repdest, self.conf['REPPY'][0], '__tmp__')
      prev = os.getcwd()
      # ----- check that files and directories in kargs exist
      for occ in kargs.values():
         obj = occ
         if not type(occ) in (ListType, TupleType):
            obj = [occ]
         for rep in obj:
            if not self.run.Exists(rep):
               self._mess(_('path does not exist : %s') % rep,
                     _('<E>_FILE_NOT_FOUND'))
      if not os.path.isdir(repdest):
         iret = self.run.MkDir(repdest, verbose=True)
      self.run.CheckOK()

      # ----- copy REPREF/aster?.exe or kargs['exe']
      iret = 0
      if kargs.has_key('exe'):
         src = kargs['exe']
      else:
         if dbg == 'debug':
            src = os.path.join(REPREF, self.conf['BIN_DBG'][0])
         else:
            src = os.path.join(REPREF, self.conf['BIN_NODBG'][0])
      dest = repdest
      if src != os.path.join(repdest, os.path.basename(src)):
         if not self.run.IsRemote(src) and self.run['symlink'] != False:
            src = self.run.PathOnly(src)
            self.run.VerbStart(_('adding a symbolic link to %s...') % src, verbose=True)
            iret = 0
            output = ''
            try:
               os.symlink(src, os.path.join(repdest, os.path.basename(src)))
            except OSError, output:
               iret = 4
               self.run.Mess(_('error occurs during creating a symbolic link to %s') % src,
                     _('<E>_SYMLINK'))
            else:
               self.run.VerbEnd(iret, output, verbose=True)
         else:
            iret = self.run.Copy(dest, src, verbose=True)
      else:
         self.run.VerbStart(_('copying %s...') % src, verbose=True)
         self.run.VerbIgnore(verbose=True)

      # ----- copy REPREF/bibpyt
      self.run.MkDir(os.path.join(repdest, self.conf['REPPY'][0], 'Cata'))
      iret = 0
      src  = os.path.join(REPREF, self.conf['SRCPY'][0], '*')
      dest = os.path.join(repdest, self.conf['REPPY'][0])
      iret = self.run.Copy(dest, src)

      # if 'py' is in kargs
      if kargs.has_key('py'):
         self._mess(_("copy user's python source files"))
         # ----- python source given by user
         lobj = kargs['py']
         if not type(lobj) in (ListType, TupleType):
            lobj = [kargs['py'], ]
         for occ in lobj:
            obj = occ
            if self.run.IsRemote(occ):
               obj = reptrav
               iret = self.run.MkDir(obj)
               if self.run.IsDir(occ):
                  src = os.path.join(occ, '*.py')
               else:
                  src = occ
               iret = self.run.Copy(obj, src, verbose=True)
               files = glob.glob(os.path.join(obj, '*.py'))
            else:
               occ = self.run.PathOnly(occ)
               if self.run.IsDir(occ):
                  files = glob.glob(os.path.join(occ, '*.py'))
               else:
                  files = [occ, ]
            for f in files:
               fo = open(f, 'r')
               lbid = fo.readline().split()
               if len(lbid)<4:
                  self._mess(_('invalid format for MODIF line : %s') % ' '.join(lbid), \
                        _('<F>_INVALID_FILE'))
               mod, rep = lbid[2:4]
               fo.close()
               dest = os.path.join(repdest, self.conf['REPPY'][0], rep, mod+'.py')
               iret = self.run.Copy(dest, f, verbose=True)

      # ----- apply PYSUPPR directives from unigest
      if kargs.has_key('unigest'):
         for f in kargs['unigest']['py']:
            self.run.Delete(os.path.join(repdest, \
                  re.sub('^'+self.conf['SRCPY'][0], self.conf['REPPY'][0], f)))

      # ----- copy REPREF/commande or kargs['cmde']
      iret = 0
      if kargs.has_key('cmde'):
         src = kargs['cmde']
      else:
         src = os.path.join(REPREF, self.conf['BINCMDE'][0])
      dest = os.path.join(repdest, self.conf['REPPY'][0], 'Cata')
      if not self.run.IsRemote(src) and self.run['symlink'] != False:
         src = self.run.PathOnly(src)
         for f in glob.glob(os.path.join(src, 'cata.py*')):
            self.run.VerbStart(_('adding a symbolic link to %s...') % f, verbose=True)
            iret = 0
            output = ''
            try:
               os.symlink(f, os.path.join(dest, os.path.basename(f)))
            except OSError, output:
               iret = 4
               self.run.Mess(_('error occurs during creating a symbolic link to %s') % f,
                     _('<E>_SYMLINK'))
            else:
               self.run.VerbEnd(iret, output, verbose=True)
      else:
         iret = self.run.Copy(dest, os.path.join(src, 'cata.py*'), verbose=True)

      # ----- copy REPREF/elements or kargs['ele']
      iret = 0
      if kargs.has_key('ele'):
         src = kargs['ele']
      else:
         src = os.path.join(REPREF, self.conf['BINELE'][0])
      if self.run.Exists(src) and src != os.path.join(repdest, 'elem.1'):
         # symlink not allowed for elem.1 (rw)
         iret = self.run.Copy(os.path.join(repdest, 'elem.1'), src, verbose=True)
      else:
         self.run.VerbStart(_('copying %s...') % src, verbose=True)
         self.run.VerbIgnore(verbose=True)

      # ----- result directories
      os.chdir(repdest)
      self.run.MkDir('RESU_ENSIGHT')
      self.run.MkDir('REPE_OUT')

      os.chdir(prev)
      self.run.Delete(reptrav)
      return iret

#-------------------------------------------------------------------------------
   def CompilCapy(self, REPREF, reptrav, cmde='', **kargs):
      """Compile commands catalog from REPREF/commande and puts result in
      'cmde'. kargs can contain :
         capy    : list of user's capy files
         unigest : GetUnigest dict with "CATSUPPR catapy module" entries
      All args can be remote.
      """
      self._mess(_('Compilation of commands catalog'), 'TITLE')
      prev = os.getcwd()
      if cmde == '':
         cmde = os.path.join(REPREF, self.conf['BINCMDE'][0])
      # ----- check that files and directories in kargs exist
      for occ in kargs.values():
         obj = occ
         if not type(occ) in (ListType, TupleType):
            obj = [occ]
         for rep in obj:
            if not self.run.Exists(rep):
               self._mess(_('path does not exist : %s') % rep,
                     _('<E>_FILE_NOT_FOUND'))
      if not os.path.isdir(cmde):
         iret = self.run.MkDir(cmde, verbose=True)
      if not os.path.isdir(reptrav):
         iret = self.run.MkDir(reptrav, verbose=True)
      self.run.CheckOK()

      # ----- work in reptrav
      os.chdir(reptrav)

      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']

      # ----- copy capy from REPREF
      if os.path.exists(self.conf['SRCCAPY'][0]):
         self.run.Delete(self.conf['SRCCAPY'][0])
      iret = self.run.Copy(self.conf['SRCCAPY'][0],
            os.path.join(REPREF, self.conf['SRCCAPY'][0]), verbose=True)

      # if 'capy' is in kargs
      if kargs.has_key('capy'):
         force = True
         self._mess(_("copy user's catalogs"))
         # ----- catapy given by user
         lobj = kargs['capy']
         if not type(obj) in (ListType, TupleType):
            lobj = [kargs['capy'], ]
         for occ in lobj:
            obj = occ
            if self.run.IsRemote(occ):
               obj = os.path.join(self.conf['SRCCAPY'][0], '__tmp__')
               iret = self.run.MkDir(obj)
               if self.run.IsDir(occ):
                  src = os.path.join(occ, '*.capy')
               else:
                  src = occ
               iret = self.run.Copy(obj, src)
               files = glob.glob(os.path.join(obj, '*.capy'))
            else:
               occ = self.run.PathOnly(occ)
               if self.run.IsDir(occ):
                  files = glob.glob(os.path.join(occ, '*.capy'))
               else:
                  files = [occ, ]
            for f in files:
               fo = open(f, 'r')
               lbid = fo.readline().split()
               if len(lbid)<2:
                  self._mess(_('invalid format for MODIF line : %s') % ' '.join(lbid), \
                        _('<F>_INVALID_FILE'))
               rep = lbid[2].lower()
               fo.close()
               dest = os.path.join(self.conf['SRCCAPY'][0], rep)
               iret = self.run.Copy(dest, f, verbose=True)
               if not rep in self.repcatapy:
                  self._mess(_('unknown module name : %s') % rep, _('<A>_ALARM'))

      # ----- apply CATSUPPR directives from unigest
      if kargs.has_key('unigest'):
         force = True
         for f in kargs['unigest']['capy']:
            self.run.Delete(os.path.join(reptrav, f))

      # ----- build cata.py
      if os.path.exists('cata.py'):
         self.run.Delete('cata.py')

      # ----- test if compilation can be skipped
      if force or not os.path.exists(os.path.join(cmde, 'cata.py')):
         force = True
      else:
         tcpy = os.stat(os.path.join(cmde, 'cata.py')).st_ctime
         ltest = glob.glob(os.path.join(self.conf['SRCCAPY'][0], '*', '*.capy'))
         for f in ltest:
            if os.stat(f).st_ctime >= tcpy:
               force = True
               break

      self.run.VerbStart(_('compilation of commands'), verbose=True)
      if not force:
         self.run.VerbIgnore(verbose=True)
      else:

         fo = open('cata.py', 'w')
         for rep in self.repcatapy:
            for capy in glob.glob(os.path.join(self.conf['SRCCAPY'][0], rep, '*.capy')):
               fo2 = open(capy, 'r')
               fo.write(fo2.read())
               fo2.close()
         fo.close()

         # ----- compile cata.py
         iret = 0
         try:
            py_compile.compile('cata.py', doraise=True)
         except py_compile.PyCompileError, msg:
            iret = 4
            self.run.VerbEnd(iret, output=msg, verbose=True)
         else:
            self.run.VerbEnd(iret, output='', verbose=True)
            kret = self.run.Copy(cmde, 'cata.py*', verbose=True)

         if iret != 0:
            self._mess(_('error during compiling cata.py'),
                  _('<E>_CATAPY_ERROR'))
         else:
            self._mess(_('Commands catalog successfully compiled'), 'OK')

      os.chdir(prev)
      self.run.Delete(reptrav)
      return iret

#-------------------------------------------------------------------------------
   def CompilEle(self, REPREF, reptrav, ele='', **kargs):
      """Compile elements catalog as 'ele' from cata_ele.pickled.
      kargs must contain arguments for a Code_Aster execution (i.e. for PrepEnv)
      and optionnaly :
         cata    : a list of user's elements files
         unigest : GetUnigest dict with "CATSUPPR catalo module" entries
         (make_surch_offi needs unigest filename)
      All args can be remote.
      """
      self._mess(_('Compilation of elements catalog'), 'TITLE')
      required = ('exe', 'cmde')
      if ele == '':
         ele = os.path.join(REPREF, self.conf['BINELE'][0])
      prev = os.getcwd()
      # ----- check for required arguments
      for a in required:
         if not kargs.has_key(a):
            self._mess('(CompilEle) '+_('argument %s is required') % a,
                  _('<E>_PROGRAM_ERROR'))
      
      # ----- check that files and directories in kargs exist
      for occ in kargs.values():
         obj = occ
         if not type(occ) in (ListType, TupleType):
            obj = [occ]
         for rep in obj:
            if not self.run.Exists(rep):
               self._mess(_('path does not exist : %s') % rep,
                     _('<E>_FILE_NOT_FOUND'))
      if not os.path.isdir(reptrav):
         iret = self.run.MkDir(reptrav, verbose=True)
      self.run.CheckOK()

      # ----- work in repobj
      os.chdir(reptrav)

      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']

      # 1. ----- if 'cata' is in kargs
      if kargs.has_key('cata'):
         force = True
         self._mess(_("copy user's catalogs"))
         fo = open('surch.cata', 'w')
         # ----- catapy given by user
         lobj = kargs['cata']
         if not type(obj) in (ListType, TupleType):
            lobj = [kargs['cata'], ]
         for occ in lobj:
            obj = occ
            if self.run.IsRemote(occ):
               obj = '__tmp__'
               iret = self.run.MkDir(obj)
               if self.run.IsDir(occ):
                  src = os.path.join(occ, '*.cata')
               else:
                  src = occ
               iret = self.run.Copy(obj, src)
               files = glob.glob(os.path.join(obj, '*.cata'))
            else:
               occ = self.run.PathOnly(occ)
               if self.run.IsDir(occ):
                  files = glob.glob(os.path.join(occ, '*.cata'))
               else:
                  files = [occ, ]
            for f in files:
               fo2 = open(f, 'r')
               self.run.VerbStart(_('adding %s') % f, verbose=True)
               fo.write(fo2.read())
               self.run.VerbEnd(iret=0, output='', verbose=True)
               fo2.close()
         fo.close()

      # 2. ----- compile elements with MAKE_SURCH_OFFI
      self.PrepEnv(REPREF, reptrav, **kargs)
      iret = 0
      cmd = [os.path.join('.', os.path.basename(kargs['exe']))]
      cmd.append(os.path.join(self.conf['REPPY'][0], self.conf['MAKE_SURCH_OFFI'][0]))
      cmd.extend(self.conf['REPPY'])
      cmd.append('MAKE_SURCH')
      cmd.append('surch.cata')
      funig = 'unigest_bidon'
      if kargs.has_key('unigest'):
         force = True
         funig = kargs['unigest']['filename']
         if self.run.IsRemote(funig):
            funig = 'unigest'
            kret = self.run.Copy(funig, kargs['unigest']['filename'])
         else:
            funig = self.run.PathOnly(funig)
      cmd.append(funig)
      cmd.append(os.path.join(REPREF, self.conf['BINPICKLED'][0]))
      cmd.append('fort.4')
      self.run.VerbStart(_('pre-compilation of elements with %s') % \
            os.path.basename(self.conf['MAKE_SURCH_OFFI'][0]), verbose=True)

      # 2.1. ----- test if compilation can be skipped
      if force or not os.path.exists(ele):
         force = True
      else:
         tele = os.stat(ele).st_ctime
         ltest = [os.path.join(REPREF, self.conf['BINPICKLED'][0]), ]
         ltest.extend(glob.glob(os.path.join(self.conf['REPPY'][0], '*', '*.py')))
         for f in ltest:
            if os.stat(f).st_ctime >= tele:
               force = True
               if self.debug:
                  print '<DBG> %s more recent than %s' % (f, ele)
               break

      if not force:
         self.run.VerbIgnore(verbose=True)
      else:
         if self.verbose:
            print
         jret, out = self.run.Shell(' '.join(cmd))
         # ----- avoid to duplicate output
         if not self.verbose:
            self.run.VerbEnd(jret, output=out, verbose=True)
         if jret != 0:
            self._mess(_('error during pre-compilation of elements'),
                  _('<F>_CATAELE_ERROR'))

      # 3. ----- build elem.1 with kargs['exe']
      cmd = [os.path.join('.', os.path.basename(kargs['exe']))]
      cmd.append(os.path.join(self.conf['REPPY'][0], self.conf['ARGPYT'][0]))
      cmd.extend(self.conf['ARGEXE'])
      cmd.append('-commandes fort.1 -mem 8 -rep none')
      fo = open('fort.1', 'w')
      fo.write("""
# COMPILATION DU CATALOGUE D'ELEMENT
DEBUT ( CATALOGUE = _F( FICHIER = 'CATAELEM' , UNITE = 4 ))
MAJ_CATA ( ELEMENT = _F())
FIN()
""")
      fo.close()
      self.run.VerbStart(_('compilation of elements with %s') \
            % kargs['exe'], verbose=True)
      
      if not force:
         self.run.VerbIgnore(verbose=True)
      else:
         if self.verbose:
            print
         jret, out = self.run.Shell(' '.join(cmd))
         # ----- avoid to duplicate output
         if not self.verbose:
            self.run.VerbEnd(jret, output=out, verbose=True)

         # ----- diagnostic of Code_Aster execution
         diag = self.GetDiag()[0]
         if self.run.GetGrav(diag)<self.run.GetGrav('<S>') \
             and os.path.exists('elem.1'):
            self._mess(_('Elements catalog successfully compiled'), diag)
         else:
            self._mess(_('error during compilation of elements'),
                  _('<F>_CATAELE_ERROR'))

         # 4. ----- copy elements to 'ele'
         iret = self.run.Copy(ele, 'elem.1', verbose=True)

      os.chdir(prev)
      self.run.Delete(reptrav)
      return iret

#-------------------------------------------------------------------------------
   def MakePickled(self, REPREF, reptrav, repdest='', **kargs):
      """Make 'repdest'/cata_ele.pickled.
      kargs can contain :
         exe     : Code_Aster executable or Python interpreter (can be remote)
      """
      self._mess(_('Prepare cata_ele.pickled'), 'TITLE')
      if repdest == '':
         repdest = REPREF
      prev = os.getcwd()
      # ----- check if reptrav exists
      if not os.path.isdir(reptrav):
         iret = self.run.MkDir(reptrav, verbose=True)

      # ----- work in repobj
      os.chdir(reptrav)

      # ----- if force is True, don't use ctime to skip a file
      force = self.run['force']

      # ----- use REPREF/aster?.exe or kargs['exe']
      iret = 0
      if kargs.has_key('exe'):
         exe = kargs['exe']
         if self.run.IsRemote(exe):
            kret = self.run.Copy(reptrav, exe)
            exe = os.path.join(reptrav, os.path.basename(exe))
         else:
            exe = self.run.PathOnly(exe)
      else:
         exe = os.path.join(REPREF, self.conf['BIN_NODBG'][0])
      
      # ----- call MAKE_CAPY_OFFI
      cmd = [exe, ]
      cmd.append(os.path.join(REPREF, self.conf['SRCPY'][0], self.conf['MAKE_CAPY_OFFI'][0]))
      cmd.append(os.path.join(REPREF, self.conf['SRCPY'][0]))
      cmd.append('TRAV_PICKLED')
      cmd.append(os.path.join(REPREF, self.conf['SRCCATA'][0]))
      cmd.extend(self.conf['BINPICKLED'])

      # ----- test if compilation can be skipped
      if not os.path.exists(os.path.join(repdest, self.conf['BINPICKLED'][0])):
         force = True
      else:
         tpick = os.stat(os.path.join(repdest, self.conf['BINPICKLED'][0])).st_ctime
         ltest = glob.glob(os.path.join(REPREF, self.conf['SRCCATA'][0], '*', '*.cata'))
         ltest.extend(glob.glob(os.path.join(REPREF, self.conf['SRCPY'][0], '*', '*.py')))
         for f in ltest:
            if os.stat(f).st_ctime >= tpick:
               force = True
               break

      self.run.VerbStart(_('build cata_ele.pickled with %s') % \
            os.path.basename(self.conf['MAKE_CAPY_OFFI'][0]), verbose=True)
      
      if not force:
         self.run.VerbIgnore(verbose=True)
      else:
         if self.verbose:
            print
         jret, out = self.run.Shell(' '.join(cmd))
         # ----- avoid to duplicate output
         if not self.verbose:
            self.run.VerbEnd(jret, output=out, verbose=True)
         if jret == 0 and os.path.exists(self.conf['BINPICKLED'][0]):
            self._mess(_('cata_ele.pickled successfully created'), 'OK')
         else:
            self._mess(_('error during making cata_ele.pickled'),
                  _('<F>_PICKLED_ERROR'))

         # ----- copy cata_ele.pickled to 'repdest'
         iret = self.run.Copy(repdest, self.conf['BINPICKLED'][0], verbose=True)

      os.chdir(prev)
      self.run.Delete(reptrav)
      return iret

#-------------------------------------------------------------------------------
   def GetUnigest(self, funig):
      """Build a dict of the file names by parsing unigest file 'funig' where
      the keys are 'f', 'f90', 'c', 'py', 'capy', 'cata', 'test' and 'fdepl'.
      funig can be remote.
      WARNING : 'f90' not yet supported !
      """
      dico = {}
      for cat in ('f', 'f90', 'c', 'py', 'capy', 'cata', 'test', 'fdepl'):
         dico[cat] = []

      # ----- keep a reference to the file for CompilEle
      dico['filename'] = funig

      # ----- check that file exists
      if not self.run.Exists(funig):
         self._mess(_('file not found : %s') % funig, _('<A>_ALARM'))
         return dico

      # ----- copy funig if it's a remote file
      if self.run.IsRemote(funig):
         name = '__tmp__.unigest'
         kret = self.run.Copy(name, funig)
      else:
         name = self.run.PathOnly(funig)

      # parse unigest
      fo = open(name, 'r')
      for line in fo:
         mat = re.search('^ *FORSUPPR +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)', line)
         if mat:
            nam = mat.group(1).lower()+'.f'
            mod = mat.group(2).lower()
            dico['f'].append(os.path.join(self.conf['SRCFOR'][0], mod, nam))
         mat = re.search('^ *F90SUPPR +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)', line)
         if mat:
            nam = mat.group(1).lower()+'.f'
            mod = mat.group(2).lower()
            dico['f'].append(os.path.join(self.conf['SRCF90'][0], mod, nam))
         mat = re.search('^ *CSUPPR +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)',
               line)
         if mat:
            nam = mat.group(1).lower()+'.c'
            mod = mat.group(2).lower()
            dico['c'].append(os.path.join(self.conf['SRCC'][0], mod, nam))
         mat = re.search('^ *PYSUPPR +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)',
               line)
         if mat:
            nam = mat.group(1)+'.py'
            mod = mat.group(2)
            dico['py'].append(os.path.join(self.conf['SRCPY'][0], mod, nam))
         mat = re.search('^ *CATSUPPR +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)', line)
         if mat:
            nam = mat.group(1).lower()
            mod = mat.group(2).lower()
            if mod in self.repcatapy:
               dico['capy'].append(os.path.join(self.conf['SRCCAPY'][0], mod, nam+'.capy'))
            elif mod in self.repcatalo:
               dico['cata'].append(os.path.join(self.conf['SRCCATA'][0], mod, nam+'.cata'))
            else:
               self._mess(_('unknown type %s in unigest file') % mod,
                     _('<A>_ALARM'))
         mat = re.search('^ *TESSUPPR +([-_a-zA-Z0-9]+)', line)
         if mat:
            nam = mat.group(1)+'.*'
            dico['test'].append(os.path.join(self.conf['SRCTEST'][0], nam))
         mat = re.search('^ *FORDEPLA +([-_a-zA-Z0-9]+) +([-_a-zA-Z0-9]+)' \
               ' +([-_a-zA-Z0-9]+)', line)
         if mat:
            nam = mat.group(1).lower()+'.f'
            old = mat.group(2).lower()
            new = mat.group(3).lower()
            dico['fdepl'].append([os.path.join(self.conf['SRCFOR'][0], old, nam),
                  os.path.join(self.conf['SRCFOR'][0], new, nam)])
      fo.close()
      return dico

#-------------------------------------------------------------------------------
   def GetDiag(self, err='fort.9', resu='fort.8'):
      """Return the diagnostic after a Code_Aster execution
      as a list : [diag, tcpu, tsys, ttotal]
         err   : error file
         resu  : result file (None to ignore it)
      """
      diag = _('<F>_ABNORMAL_ABORT')
      tcpu = '0.'
      tsys = '0.'
      ttot = '0.'
      # OK
      ok = False
      # <A>_ALARME
      alarm = False
      # <F>_VERSION_EXPIREE
      expir = False
      # <F>_SUPERVISEUR
      superv = False
      # <F>_ERREUR
      fatal = False
      # <S>_ARRET_CPU (compatible avec OK)
      arretcpu = False
      # <S>_MEMOIRE (compatible avec OK)
      mem = False
      # <S>_ERREUR (compatible avec OK)
      errs = False
      
      # 1. ----- parse error file
      if err == None or not os.path.exists(err):
         err = resu
      if not os.path.exists(err):
         #self._mess(_('file not found %s') % err, '<E>_FICHIER_ERRE')
         pass
      else:
         fo = open(err, 'r')
         for line in fo:
            if line.find(
            """<I> <FIN> ARRET NORMAL DANS "FIN" PAR APPEL A "JEFINI".""") > -1:
               ok = True
            elif line.find('<A>') > -1:
               alarm = True
            elif line.find('<F> <INIAST> VERSION EXPIREE') > -1:
               expir = True
            elif line.find('<F> <SUPERVISEUR> ARRET SUR ERREUR(S) UTILISATEUR') > -1:
               superv = True
            elif line.find('<F>') > -1:
               fatal = True
            elif line.find('MEMOIRE INSUFFISANTE POUR ALLOUER') > -1:
               errs = True
               mem = True
            elif line.find('ARRET PAR MANQUE DE TEMPS CPU') > -1:
               errs = True
               arretcpu = True
            elif line.find('<S>') > -1:
               errs = True
         fo.close()
         if ok:
            diag = 'OK'
            if alarm:
               diag = _('<A>_ALARM')
         # fatal error
         if fatal:
            diag = _('<F>_ERROR')
            if superv:
               diag = _('<F>_SUPERVISOR')
            elif expir:
               diag = _('<F>_EXPIRED_VERS')
         # "S" (recoverable) error
         if errs:
            diag = _('<S>_ERROR')
            if arretcpu:
               diag = _('<S>_CPU_LIMIT')
            elif mem:
               diag = _('<S>_MEMORY_ERROR')
         if not ok or errs or fatal:
            return diag, tcpu, tsys, ttot

      # 2. ----- parse result file
      if not os.path.exists(resu):
         self._mess(_('file not found %s') % resu, _('<E>_NO_RESU_FILE'))
      else:
         # OK / NOOK
         ok = False
         nook = False
         # <A>_ALARME
         alarm = False
         diag0 = diag
         fo = open(resu, 'r')
         for line in fo:
            if re.search('^ *NOOK +', line):
               nook = True
            elif re.search('^ *OK +', line):
               ok = True
            if line.find('<A>') > -1:
               alarm = True
            if line.find('TOTAL_JOB') > -1:
               tcpu = line.split()[3]
               tsys = line.split()[5]
               ttot = line.split()[7]
         fo.close()
         if nook:
            diag0 = 'NOOK_TEST_RESU'
         else:
            diag0 = 'OK'
            if alarm:
               diag0 = _('<A>_ALARM')
         diag = diag0
      
      return diag, tcpu, tsys, ttot


